#include "Collision.h"
#include "mathf.h"

bool PointCercle(Point point, Cercle cercle)
{
	if ((point.x-cercle.x)*(point.x-cercle.x) + (point.y-cercle.y)*(point.y-cercle.y) > cercle.rayon*cercle.rayon)
	{
		return false;
	}
	else
	{
		return true;
	}
}

bool DroiteCercle(Segment segment,Cercle cercle)
{
	Vecteur u;
	Vecteur AC;
	float numerateur;
	float denominateur;
	float CI ;
	
	u.x = segment.x2 - segment.x1;
	u.y = segment.y2 - segment.y1;
	AC.x = cercle.x - segment.x1;
	AC.y = cercle.y - segment.y1;
	numerateur = fabsf(u.x*AC.y - u.y*AC.x);   // norme du vecteur v
	denominateur = sqrtf(u.x*u.x + u.y*u.y);  // norme de u
	CI = numerateur / denominateur;
	if (CI<cercle.rayon)
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool SegmentCercle(Segment segment,Cercle cercle)
{
	Vecteur AB, AC, BC;
	Point point1, point2;
	float pscal1, pscal2;
	
	if (DroiteCercle(segment, cercle) == false)
	{
		return false;  // si on ne touche pas la droite, on ne touchera jamais le segment
	}
	point1.x = segment.x1;
	point1.y = segment.y1;
	point2.x = segment.x2;
	point2.y = segment.y2;
	AB.x = segment.x2 - segment.x1;
	AB.y = segment.y2 - segment.y1;
	AC.x = cercle.x - segment.x1;
	AC.y = cercle.y - segment.y1;
	BC.x = cercle.x - segment.x2;
	BC.y = cercle.y - segment.y2;
	pscal1 = AB.x*AC.x + AB.y*AC.y;  // produit scalaire
	pscal2 = (-AB.x)*BC.x + (-AB.y)*BC.y;  // produit scalaire
	if (pscal1>=0 && pscal2>=0)
	{
		return true;   // I entre A et B, ok.
	}
	// dernire possibilit, A ou B dans le cercle
	if (PointCercle(point1, cercle))
	{
		return true;
	}
	if (PointCercle(point2, cercle))
	{
		return true;
	}
	return false;
}

Point SegmentProjectionPoint(Segment segment,Point point)
{
  Vecteur u,AC;
  float ti;
  Point I;
  
  u.x = segment.x2 - segment.x1; 
  u.y = segment.y2 - segment.y1;
  AC.x = point.x - segment.x1;
  AC.y = point.y - segment.y1;
  ti = (u.x*AC.x + u.y*AC.y) / (u.x*u.x + u.y*u.y);
  I.x = segment.x1 + ti*u.x;
  I.y = segment.y1 + ti*u.y;
  return I;
}

Vecteur NormaleSegment(Segment segment,Point point)
{
  Vecteur AC,u,N;
  float parenthesis, norme;
  
  u.x = segment.x2 - segment.x1;  
  u.y = segment.y2 - segment.y1;
  AC.x = point.x - segment.x1;  
  AC.y = point.y - segment.y1;
  parenthesis = u.x*AC.y-u.y*AC.x;  // calcul une fois pour les deux
  N.x = -u.y*(parenthesis);
  N.y = u.x*(parenthesis);
  // normalisons
  norme = sqrtf(N.x*N.x + N.y*N.y);
  N.x/=norme;
  N.y/=norme;
  return N;
}

Vecteur Rebond(Vecteur vecteur,Vecteur normale)
{
	Vecteur v2;
	float pscal;
	
	pscal = (vecteur.x*normale.x +  vecteur.y*normale.y);
	v2.x = vecteur.x -2*pscal*normale.x;
	v2.y = vecteur.y -2*pscal*normale.y;
	return v2;
}

bool DroiteSegment(Segment segment1,Segment segment2)
{
	Vecteur AO,AP,AB;
	
	AB.x = segment1.x2 - segment1.x1;
	AB.y = segment1.y2 - segment1.y1;
	AP.x = segment2.x2 - segment1.x1;
	AP.y = segment2.y2 - segment1.y1;
	AO.x = segment2.x1 - segment1.x1;
	AO.y = segment2.y1 - segment1.y1;
	if ((AB.x*AP.y - AB.y*AP.x)*(AB.x*AO.y - AB.y*AO.x) < 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool SegmentSegment(Segment segment1,Segment segment2)
{
	if (DroiteSegment(segment1,segment2) == false)
	{
		return false;
	}
	if (DroiteSegment(segment2,segment1) == false)
	{
		return false;
	}
	return true;
}